@@ -175,7 +175,7 @@ class W(object):
         """Reset properties."""
         self._cache = {}
 
-    def to_file(self, path='', format=None):
+    def to_file(self, path="", format=None):
         """
         Write a weights to a file. The format is guessed automatically 
         from the path, but can be overridden with the format argument. 
@@ -194,13 +194,12 @@ class W(object):
         -------
         None
         """
-        f = popen(dataPath=path, mode='w', dataFormat=format)
+        f = popen(dataPath=path, mode="w", dataFormat=format)
         f.write(self)
         f.close()
-        
 
     @classmethod
-    def from_file(cls, path='', format=None):
+    def from_file(cls, path="", format=None):
         """
         Read a weights file into a W object. 
 
@@ -215,7 +214,7 @@ class W(object):
         -------
         W object
         """
-        f = popen(dataPath=path, mode='r', dataFormat=format)
+        f = popen(dataPath=path, mode="r", dataFormat=format)
         w = f.read()
         f.close()
         return w
@@ -877,9 +876,9 @@ class W(object):
             for j, neigh_list in list(self.neighbors.items()):
                 self.__neighbors_0[j] = [id2i[neigh] for neigh in neigh_list]
             self._cache["neighbors_0"] = self.__neighbors_0
-        
+
         neighbor_list = self.__neighbors_0
-        
+
         return neighbor_list
 
     def get_transform(self):
@@ -1353,7 +1352,7 @@ class WSP(object):
 
     """
 
-    def __init__(self, sparse, id_order=None):
+    def __init__(self, sparse, id_order=None, index=None):
         if not scipy.sparse.issparse(sparse):
             raise ValueError("must pass a scipy sparse object")
         rows, cols = sparse.shape
@@ -1367,6 +1366,18 @@ class WSP(object):
                     "Number of values in id_order must match shape of sparse"
                 )
         self.id_order = id_order
+        # temp addition of index attribute
+        import pandas as pd  # will be removed after refactoring is done
+        if index is not None:
+            if not isinstance(index, (pd.Index, pd.MultiIndex, pd.RangeIndex)):
+                raise TypeError("index must be an instance of pandas.Index dtype")
+            if len(index) != self.n:
+                raise ValueError(
+                    "Number of values in index must match shape of sparse"
+                )
+        else:
+            index = pd.RangeIndex(self.n)
+        self.index = index
         self._cache = {}
 
     @property
